@@ -33,6 +33,7 @@
 #include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
 #include "typedefs.h"
 #include "main.h"
@@ -42,12 +43,10 @@
 #include "goo-window.h"
 #include "goo-stock.h"
 #include "gth-image-list.h"
-#include "gnome-vfs-helpers.h"
 #include "preferences.h"
 #include "gconf-utils.h"
-#include "gnome-vfs-helpers.h"
 
-#define GLADE_CHOOSER_FILE "goo_cover_chooser.glade"
+#define GLADE_CHOOSER_FILE "cover_chooser.glade"
 #define MAX_IMAGES 20
 #define READ_TIMEOUT 20
 #define BUFFER_SIZE 612
@@ -76,6 +75,7 @@ struct _DialogData {
 	gboolean             autofetching;
 
 	FileSavedFunc        file_saved_func;
+	char                *source;
 	char                *dest;
 
 	GnomeVFSAsyncHandle *vfs_handle;
@@ -88,20 +88,31 @@ struct _DialogData {
 	GtkWidget           *progress_label;
 	GtkWidget           *ok_button;
 	GtkWidget           *revert_button;
+	GtkWidget           *cc_cancel_search_button;
 };
 
 
-/* called when the main dialog is closed. */
+static void
+cancel_search (DialogData *data)
+{
+	if (data->vfs_handle != NULL) {
+		gnome_vfs_async_cancel (data->vfs_handle);
+		data->vfs_handle = NULL;
+	}
+
+	if (data->load_id != 0) {
+		g_source_remove (data->load_id);
+		data->load_id = 0;
+	}
+}
+
+
 static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	if (data->vfs_handle != NULL)
-		gnome_vfs_async_cancel (data->vfs_handle);
-
-	if (data->load_id != 0)
-		g_source_remove (data->load_id);
-
+	cancel_search (data);
+	
 	if (data->tmpdir != NULL) {
 		g_list_foreach (data->tmpfiles, (GFunc) gnome_vfs_unlink, NULL);
 		path_list_free (data->tmpfiles);
@@ -111,6 +122,7 @@ destroy_cb (GtkWidget  *widget,
 	}
 
 	g_free (data->dest);
+	g_free (data->source);
 	g_free (data->album);
 	g_free (data->artist);
 	path_list_free (data->url_list);
@@ -133,15 +145,8 @@ copy_progress_update_cb (GnomeVFSAsyncHandle      *handle,
 	if (info->status != GNOME_VFS_XFER_PROGRESS_STATUS_OK) {
 		data->vfs_result = info->status;
 		return FALSE;
-
-	} else if (info->phase == GNOME_VFS_XFER_PHASE_OPENTARGET) {
-		debug (DEBUG_INFO, "OPEN TARGET: %s\n", info->target_name);
-		data->tmpfiles = g_list_prepend (data->tmpfiles, g_strdup (info->target_name));
-
-	} else if (info->phase == GNOME_VFS_XFER_PHASE_CLOSETARGET) {
-		debug (DEBUG_INFO, "CLOSE TARGET: %s\n", info->target_name);
-
-	} else if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
+	} 
+	if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
 		debug (DEBUG_INFO, "COMPLETED");
 	
 		if (data->file_saved_func != NULL)
@@ -154,8 +159,8 @@ copy_progress_update_cb (GnomeVFSAsyncHandle      *handle,
 
 static void
 copy_file_from_url (DialogData    *data,
-		    const char    *src,
-		    const char    *dest,
+		    const char    *uri_source,
+		    const char    *uri_dest,
 		    FileSavedFunc  file_saved_func)
 {
 	GList                     *src_uri_list, *dest_uri_list;
@@ -165,11 +170,12 @@ copy_file_from_url (DialogData    *data,
 	GnomeVFSResult             result;
 
 	data->file_saved_func = file_saved_func;
+	
 	g_free (data->dest);
-	data->dest = g_strdup (dest);
+	data->dest = g_strdup (uri_dest);
 
-	src_uri_list = g_list_prepend (NULL, new_uri_from_path (src));
-	dest_uri_list = g_list_prepend (NULL, new_uri_from_path (dest));
+	src_uri_list = g_list_prepend (NULL, gnome_vfs_uri_new (uri_source));
+	dest_uri_list = g_list_prepend (NULL, gnome_vfs_uri_new (uri_dest));
 
 	xfer_options    = GNOME_VFS_XFER_DEFAULT;
 	xfer_error_mode = GNOME_VFS_XFER_ERROR_MODE_ABORT;
@@ -199,7 +205,7 @@ copy_file_from_url (DialogData    *data,
 
 	if (result != GNOME_VFS_OK) {
 		if (file_saved_func != NULL)
-			(*file_saved_func) (data, dest, FALSE);
+			(*file_saved_func) (data, uri_dest, FALSE);
 	}
 }
 
@@ -332,7 +338,7 @@ make_file_list_from_search_result (DialogData *data,
 		return FALSE;
 	
 	partial_url = NULL;
-	while ((n = read (fd, buf+buf_offset, BUFFER_SIZE-buf_offset-1)) > 0) {
+	while ((n = read (fd, buf + buf_offset, BUFFER_SIZE - buf_offset - 1)) > 0) {
 		const char *prefix = "/images?q=tbn:";
 		int         prefix_len = strlen (prefix);
 		char       *url_start;
@@ -357,8 +363,8 @@ make_file_list_from_search_result (DialogData *data,
 					g_string_append (partial_url, url_start);
 				url_start = NULL;
 				copy_tail = FALSE;
-
-			} else {
+			}
+			else {
 				char *url_tail = g_strndup (url_start, url_end - url_start);
 				char *url;
 				char *complete_url;
@@ -369,7 +375,8 @@ make_file_list_from_search_result (DialogData *data,
 					url = partial_url->str;
 					g_string_free (partial_url, FALSE);
 					partial_url = NULL;
-				} else 
+				} 
+				else 
 					url = url_tail;
 					
 				complete_url = g_strconcat ("http://images.google.com", url, NULL);
@@ -396,7 +403,8 @@ make_file_list_from_search_result (DialogData *data,
 				 buf + buf_offset + n - prefix_len, 
 				 prefix_len);
 			buf_offset = prefix_len;
-		} else 
+		} 
+		else 
 			buf_offset = 0;
 	}
 
@@ -421,7 +429,7 @@ search_result_saved_cb (DialogData *data,
 {
 	if (! success) {
 		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
-				       _("Could not search a cover on Internet"),
+				       _("Could not search for a cover on Internet"),
 				       gnome_vfs_result_to_string (data->vfs_result));
 		return;
 	}
@@ -439,13 +447,15 @@ search_result_saved_cb (DialogData *data,
 static char*
 get_query (DialogData *data)
 {
-	return g_strconcat ("http://images.google.com",
-			    "/images?q=",
-			    data->album,
-			    "+",
-			    data->artist,
-			    /* "&imgsz=medium", FIXME*/
-			    NULL);
+	char *s, *e, *q;
+
+	s = g_strdup_printf ("\"%s\" \"%s\"", data->album, data->artist);
+	e = gnome_vfs_escape_string (s);
+	q = g_strconcat ("http://images.google.com/images?q=", e, NULL);
+	g_free (e);
+	g_free (s);
+	
+	return q;
 }
 
 
@@ -516,7 +526,7 @@ revert_cb (GtkWidget  *widget,
 		return;
 
 	original_cover = goo_window_get_cover_filename (data->window);
-	gnome_vfs_x_copy_uri_simple (data->cover_backup, original_cover);
+	file_copy (data->cover_backup, original_cover);
 	goo_window_update_cover (data->window);
 }
 
@@ -565,9 +575,6 @@ image_list_item_activated_cb (GthImageList *list,
 }
 
 
-
-
-
 static void
 backup_cover_image (DialogData   *data)
 {
@@ -590,7 +597,7 @@ backup_cover_image (DialogData   *data)
 	gnome_vfs_uri_unref (uri);
 
 	cover_backup = g_build_filename (data->tmpdir, ORIGINAL_COVER, NULL);
-	gnome_vfs_x_copy_uri_simple (original_cover, cover_backup);
+	file_copy (original_cover, cover_backup);
 	g_free (original_cover);
 
 	data->tmpfiles = g_list_prepend (data->tmpfiles, cover_backup);
@@ -598,6 +605,22 @@ backup_cover_image (DialogData   *data)
 }
 
 
+static void
+cancel_search_button_clicked_cb (GtkWidget  *widget, 
+       				 DialogData *data)
+{
+	char *text;
+	
+	cancel_search (data);
+	
+	text = g_strdup_printf ("%u", data->urls);
+	gtk_label_set_text (GTK_LABEL (data->progress_label), text);
+	g_free (text);
+	
+	gtk_widget_set_sensitive (widget, FALSE);
+}
+
+
 void
 dlg_cover_chooser (GooWindow  *window,
 		   const char *album,
@@ -632,6 +655,8 @@ dlg_cover_chooser (GooWindow  *window,
 	btn_cancel = glade_xml_get_widget (data->gui, "cc_cancelbutton");
 	btn_help = glade_xml_get_widget (data->gui, "cc_helpbutton");
 
+	data->cc_cancel_search_button = glade_xml_get_widget (data->gui, "cc_cancel_search_button");
+
 	data->image_list = gth_image_list_new (THUMB_SIZE + THUMB_BORDER);
 	gth_image_list_set_view_mode (GTH_IMAGE_LIST (data->image_list),
 				      GTH_VIEW_MODE_VOID);
@@ -680,6 +705,10 @@ dlg_cover_chooser (GooWindow  *window,
 			  "item_activated",
 			  G_CALLBACK (image_list_item_activated_cb),
 			  data);
+	g_signal_connect (G_OBJECT (data->cc_cancel_search_button), 
+			  "clicked",
+			  G_CALLBACK (cancel_search_button_clicked_cb),
+			  data);
 
 	/* run dialog. */
 
@@ -700,9 +729,6 @@ dlg_cover_chooser (GooWindow  *window,
 }
 
 
-
-
-
 static void
 auto_fetch__image_saved_cb (DialogData *data,
 			    const char *filename,
@@ -710,10 +736,11 @@ auto_fetch__image_saved_cb (DialogData *data,
 {
 	if (success) {
 		char *tmpfile = g_strdup (filename);
+		
 		debug (DEBUG_INFO, "LOAD IMAGE: %s\n", tmpfile);
 		data->tmpfiles = g_list_prepend (data->tmpfiles, tmpfile);
 
-		goo_window_set_cover_image (data->window, filename);
+		goo_window_set_cover_image (data->window, get_path_from_uri (filename));
 	}
 
 	destroy_cb (NULL, data);
@@ -738,14 +765,14 @@ auto_fetch__search_result_saved_cb (DialogData *data,
 		g_free (filename);
 
 		copy_file_from_url (data, url, dest, auto_fetch__image_saved_cb);
-
-	} else
+	} 
+	else
 		destroy_cb (NULL, data);
 }
 
 
 static gboolean
-auto_fetch__start_searching (gpointer callback_data)
+auto_fetch_from_name__start_searching (gpointer callback_data)
 {
 	DialogData *data = callback_data;
 	char       *query;
@@ -766,9 +793,9 @@ auto_fetch__start_searching (gpointer callback_data)
 
 
 void
-fetch_cover_image (GooWindow  *window,
-		   const char *album,
-		   const char *artist)
+fetch_cover_image_from_name (GooWindow  *window,
+		             const char *album,
+		             const char *artist)
 {
 	DialogData *data;
 	
@@ -782,5 +809,41 @@ fetch_cover_image (GooWindow  *window,
 	data->tmpdir = g_strdup (get_temp_work_dir ());
 	ensure_dir_exists (data->tmpdir, DIR_PERM);
 
-	data->load_id = g_idle_add (auto_fetch__start_searching, data);
+	data->load_id = g_idle_add (auto_fetch_from_name__start_searching, data);
+}
+
+
+static gboolean
+auto_fetch_from_asin__start_searching (gpointer callback_data)
+{
+	DialogData *data = callback_data;
+	char       *dest;
+
+	g_source_remove (data->load_id);
+	data->load_id = 0;
+
+	dest = g_strconcat ("file://", data->tmpdir, "/1.jpg", NULL);
+	copy_file_from_url (data, data->source, dest, auto_fetch__image_saved_cb);
+	g_free (dest);
+
+	return FALSE;
+}
+
+
+void
+fetch_cover_image_from_asin (GooWindow  *window,
+		             const char *asin)
+{
+	DialogData *data;
+	
+	data = g_new0 (DialogData, 1);
+	data->window = window;
+	data->max_images = 1;
+	data->autofetching = TRUE;
+
+	data->tmpdir = g_strdup (get_temp_work_dir ());
+	ensure_dir_exists (data->tmpdir, DIR_PERM);
+
+	data->source = g_strdup_printf ("http://images.amazon.com/images/P/%s.01._SCLZZZZZZZ_.jpg", asin);
+	data->load_id = g_idle_add (auto_fetch_from_asin__start_searching, data);
 }
